! Construit les cartes qui servent dans le recul de grounding line
! et teste les methodes de recul.
! la diminution de H est lineaire avec le recul

module proto_recul_mod

use io_netcdf_grisli
use declar_proto_recul
implicit none


contains   

!----------------------------------------------------------------------------------
! subroutine  time_step_recu       : ce qui se fait a chaque pas de temps
! subroutine  init_proto_recul     : lit les donnees et initialise
! subroutine  init_masques         : initialise les masques
! subroutine  pos_gr_line_init     : Calcule les positions initiales de Grounding line
! subroutine  update_retreat_rates : en fonction du temps
! subroutine  calc_delHdt_maj      : calcul le dHdt et le new H sur chaque maille
! subroutine  calc_H_float         : calcule la hauteur de flottaison
! subroutine  update_flottants     :  points qui deviennent flottant
! subroutine  update_a_traiter     :  points qui deviennent a traiter
! subroutine  prescribe_Hp         :  renvoie Hp et I_Hp vers GRISLI
!----------------------------------------------------------------------------------

!----------------------------------------------------------------------------------
! time_step_recul : ce qui se fait a chaque pas de temps
!----------------------------------------------------------------------------------

subroutine time_step_recul

implicit none

  call update_retreat_rates      !  update les vitesses de retrait (autorise que si time > time_dep)
  Btest(:,:) = Bsoc(:,:) + Bsoc_sigma(:,:)
  call calc_eps_max              !  pour le sanity check
  call calc_delHdt_maj           !  calcule delHdt et nouveau H pour chaque point 
  call calc_H_float              !  calcule la valeur de H_flot
  call update_flottants          !  update les nouveaux points flottants et xgl, delHdx associes
  call prescribe_Hp              !  renvoie Hp et I_Hp vers GRISLI
  call update_a_traiter          !  update le masque des points a traiter

end subroutine time_step_recul



!----------------------------------------------------------------------------------
! init_proto_recul : lit les donnees et initialise
!----------------------------------------------------------------------------------

subroutine init_proto_recul

use declar_proto_recul
implicit none


 namelist/retreat/number_data_B_file,region_file,file_exp,bed_sigma_1,bed_sigma_2


    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
428 format(A)
    read(num_param,retreat)

    write(num_rep_42,428) '!___________________________________________________________' 
    write(num_rep_42,428) '!  read parameters of retreat'
    write(num_rep_42,retreat)                    
    write(num_rep_42,428) '!___________________________________________________________' 

  call lect_retreat_files


  call init_masques                     ! initialise mk_traiter, retreat0_x, retreat0_y, Deltatot_H0,
  call pos_gr_line_init                 ! calcule x_gl_mx,  y_gl_my, delHdx_mx, delHdy_my a l'initialisation
                                        ! ensuite cette position est updated dans la boucle

  call calc_eps_max
  delHdt_sanity(:,:) = 5000.


end subroutine init_proto_recul





!-------------------------------------------------------------------------
!<  init_masques : initialise les masques
!-------------------------------------------------------------------------
subroutine init_masques

!--------------------------------------------------------------------------
! Pour chaque noeud pose dont le socle est sous le niveau des mers
!
! definition des noeuds a traiter
!------------------------------------
! 
! mk_traiter = 2 : pose loin de gl
! mk_traiter = 1 : en cours de recul
! mk_traiter = -1: flottant implique dans le recul
! mk_traiter = -2: flottant loin de GL 
! mk_traiter = 0 : pas a traiter

  use declar_proto_recul
  implicit none
  integer                           :: som_voisins  !< pour des tests


! definition du masque a traiter

  where ((Mk_gr(:,:).eq.1).and.(Bsoc(:,:).lt.0.))      ! points  traiter
     mk_traiter(:,:) = 1
     H_float(:,:)    = Bsoc(:,:) / Coef_bflot
  elsewhere ((Mk_gr(:,:).eq.1).and.(Bsoc(:,:).ge.0.))  ! pose et stable
     mk_traiter(:,:) = 0
     H_float(:,:)     = -10000.  
  elsewhere (Mk_gr(:,:).eq.0)                          
     mk_traiter(:,:) = -1
     H_float   (:,:) = Bsoc(:,:) / Coef_bflot
  end where


  do j=2,ny-1
     do i =2,nx-1  

        if (mk_traiter(i,j).eq.1) then              ! pose et a traiter
           if (any(mk_gr(i-1:i+1,j-1:j+1).eq.0)) then
              mk_traiter(i,j) = 1                   ! tout de suite
           else
              mk_traiter(i,j) = 2                   ! apres propagation
           end if
        end if

        if (mk_traiter(i,j).eq.-1) then 
           if (any(mk_gr(i-1:i+1,j-1:j+1).eq.1)) then
              mk_traiter(i,j) = -1                  ! proche de la grounding line
           else
              mk_traiter(i,j) = -2                  ! loin de la grounding line
           end if
        end if
   
     end do
  end do


call init_retreat0

call init_time_depart

call update_retreat_rates                  ! retreat rate depend du temps

call bedrock_sigma





mk_traiter0(:,:) = mk_traiter(:,:)

end subroutine init_masques



!---------------------------------------------------------------------------
!< pos_gr_line : Calcule les positions initiales de Grounding line
!---------------------------------------------------------------------------
subroutine pos_gr_line_init

! Calcul des positions de Grounding line
!__________________________________________
!
! les points sont sur les mailles mineures. 
! la coordonnee est positive quand le point grounded est  l'Est ou au Nord
!
!         ~--------------x------------o            
!         i-1                         i                 x_gl_mx >0
!        float                      grounded
!  <-  West                               -> East     
!
! si deux points flottants:  x_gl_mx = -10 dx
! si deux points poses :      x_gl_mx = 10 dx

  use declar_proto_recul
  implicit none

  ! variables locales
  real      :: H_0         ! epaisseur en amont gl
  real      :: H_1         ! epaisseur en aval gl
  real      :: B_0         ! socle en amont gl
  real      :: B_1         ! socle en aval gl
  real      :: dyy         ! variable de travail  longueur de la maille en diagonale       

  dyy = dx*(2.**0.5)       !longueur de la diagonale

  do j=2,ny-1
     do i=2,nx-1   
      
!     recherche position sous maille  selon x -------------------------------------------

        x_gl: if ((mk_gr(i,j).eq.1).and.(mk_gr(i-1,j).eq.0)) then      ! grounded a l'Est

           H_1 = H(i-1,j)
           H_0 = H(i,j)
           B_1 = Bsoc(i-1,j)
           B_0 = Bsoc(i,j)

           call calc_xgl(dx,coef_Bflot,H_0,B_0,H_1,B_1, x_gl_mx(i,j),delHdx_mx(i,j))  

        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i-1,j).eq.1)) then       ! grounded a l'West

           H_1 = H(i,j)
           H_0 = H(i-1,j)
           B_1 = Bsoc(i,j)
           B_0 = Bsoc(i-1,j)

           call calc_xgl(dx,coef_Bflot,H_0,B_0,H_1,B_1, x_gl_mx(i,j),delHdx_mx(i,j))  

           x_gl_mx(i,j)   = - x_gl_mx(i,j)                            ! on change le signe de x_gl
         

        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i-1,j).eq.0)) then      ! tout le monde flottant

           x_gl_mx(i,j)   = -10.* dx
           delHdx_mx(i,j) =  0.


        else                                                          ! tout le monde pose

           x_gl_mx(i,j)   = 10.*dx
           delHdx_mx(i,j) = 0.

        end if x_gl

!     recherche position sous maille  selon y -------------------------------------------

        y_gl: if ((mk_gr(i,j).eq.1).and.(mk_gr(i,j-1).eq.0)) then      ! grounded au Nord

           H_1 = H(i,j-1)
           H_0 = H(i,j)
           B_1 = Bsoc(i,j-1)
           B_0 = Bsoc(i,j)

           call calc_xgl(dx,coef_Bflot,H_0,B_0,H_1,B_1, y_gl_my(i,j),delHdy_my(i,j))  


        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i,j-1).eq.1)) then       ! grounded au Sud 

           H_1 = H(i,j)
           H_0 = H(i,j-1)
           B_1 = Bsoc(i,j)
           B_0 = Bsoc(i,j-1)

           call calc_xgl(dx,coef_Bflot,H_0,B_0,H_1,B_1, y_gl_my(i,j),delHdy_my(i,j))  

           y_gl_my(i,j)   = - y_gl_my(i,j)                            ! on change le signe de y_gl
         

        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i,j-1).eq.0)) then      ! tout le monde flottant

           y_gl_my(i,j)   = -10.*dx
           delHdy_my(i,j) = 0.


        else                                                          ! tout le monde pose

           y_gl_my(i,j)   = 10.*dx
           delHdy_my(i,j) = 0.

        end if y_gl

!     recherche position sous maille  selon diagonale SW-NE -------------------------------------------

        SW_gl: if ((mk_gr(i,j).eq.1).and.(mk_gr(i-1,j-1).eq.0)) then      ! grounded au nord-est

           H_1 = H(i-1,j-1)
           H_0 = H(i,j)
           B_1 = Bsoc(i-1,j-1)
           B_0 = Bsoc(i,j)

           call calc_xgl(dyy,coef_Bflot,H_0,B_0,H_1,B_1, SW_gl_m(i,j),delHdx_SW(i,j))  


        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i-1,j-1).eq.1)) then       ! grounded au Sud-west 

           H_1 = H(i,j)
           H_0 = H(i-1,j-1)
           B_1 = Bsoc(i,j)
           B_0 = Bsoc(i-1,j-1)

           call calc_xgl(dyy,coef_Bflot,H_0,B_0,H_1,B_1, SW_gl_m(i,j),delHdx_SW(i,j))  

           SW_gl_m(i,j) = - SW_gl_m(i,j)                                 ! on change le signe de SW_gl
         

        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i-1,j-1).eq.0)) then      ! tout le monde flottant

           SW_gl_m(i,j)   = -10.*dx
           delHdx_SW(i,j) = 0.


        else                                                          ! tout le monde pose

           SW_gl_m(i,j)   = 10.*dx
           delHdx_SW(i,j) = 0.


        end if SW_gl

!     recherche position sous maille  selon diagonale SE-NW -------------------------------------------


        SE_gl: if ((mk_gr(i,j).eq.1).and.(mk_gr(i+1,j-1).eq.0)) then      ! grounded au nord-west

           H_1 = H(i+1,j-1)
           H_0 = H(i,j)
           B_1 = Bsoc(i+1,j-1)
           B_0 = Bsoc(i,j)

           call calc_xgl(dyy,coef_Bflot,H_0,B_0,H_1,B_1,SE_gl_m(i,j),delHdx_SE(i,j))  


        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i+1,j-1).eq.1)) then       ! grounded au Sud-Est 

           H_1 = H(i,j)
           H_0 = H(i+1,j-1)
           B_1 = Bsoc(i,j)
           B_0 = Bsoc(i+1,j-1)

           call calc_xgl(dyy,coef_Bflot,H_0,B_0,H_1,B_1,SE_gl_m(i,j),delHdx_SE(i,j))  
         

           SE_gl_m(i,j) = - SE_gl_m(i,j)                                 ! on change le signe de SE_gl
         

        else if ((mk_gr(i,j).eq.0).and.(mk_gr(i+1,j-1).eq.0)) then       ! tout le monde flottant

           SE_gl_m(i,j)   = -10.*dx
           delHdx_SE(i,j) = 0.


        else                                                              ! tout le monde pose

           SE_gl_m(i,j)   = 10.*dx
           delHdx_SE(i,j) = 0.



        end if SE_gl

      
     end do
  end do

  return
end subroutine pos_gr_line_init



!-------------------------------------------------------------------------
!<  update_retreat_rates  : en fonction du temps
!-------------------------------------------------------------------------

subroutine update_retreat_rates

  use declar_proto_recul
  implicit none


  where (time_dep_mx(:,:).lt.time)                                      
     retreat_x(:,:) =  retreat0_x(:,:)
  elsewhere
      retreat_x(:,:) = 0.
   end where

   where (time_dep_my(:,:).lt.time)                                      
      retreat_y(:,:) =  retreat0_y(:,:)
   elsewhere
      retreat_y(:,:) = 0.
   end where

  where (time_dep_SW(:,:).le.time)                                      
     retreat_SW(:,:) =  retreat0_SW(:,:)
  elsewhere
     retreat_SW(:,:) = 0.
  end where

  where (time_dep_SE(:,:).le.time)                                      
     retreat_SE(:,:) =  retreat0_SE(:,:)
  elsewhere
     retreat_SE(:,:) = 0.
  end where




   return
 end subroutine update_retreat_rates



!-------------------------------------------------------------------------
!<  calc_delHdt_maj  : calcul le dHdt et le new H sur chaque maille
!-------------------------------------------------------------------------

subroutine calc_delHdt_maj

  use declar_proto_recul
  implicit none

  real,dimension(2)                  :: dH_x_voisin   ! quand on balaye les voisins
  real,dimension(2)                  :: dH_y_voisin   ! quand on balaye les voisins
  real,dimension(2)                  :: dH_SW_voisin  ! quand on balaye les voisins
  real,dimension(2)                  :: dH_SE_voisin  ! quand on balaye les voisins
  real                               :: max_x         ! valeur max de  dH_x_voisin 
  real                               :: max_y         ! valeur max de  dH_y_voisin 
  real                               :: max_SW        ! valeur max de  dH_SW_voisin 
  real                               :: max_SE        ! valeur max de  dH_SE_voisin 
  integer                            :: som_voisins   !< pour des tests

! conditions grounding line du point de vue du noeud majeur i

!  ~--------------x--------------o---------------x-----------------~
!  i-1                           i                                i+1
!         0 < x_gl(i) < 10 dx          -10 dx < x_gl(i+1) < 0

 
  do j=2,ny-1
     do i=2,nx-1
       
        point_a_traiter:   if (mk_traiter(i,j).eq.1) then                ! Point  traiter 

           som_voisins     = 0                                           ! balaye les voisins en croix
           dH_x_voisin(:)  = 0.                                                         
           dH_y_voisin(:)  = 0.                                          

           dH_SW_voisin(:) = 0.                                          ! et en diagonale
           dH_SE_voisin(:) = 0.                                          ! 


           if ((mk_traiter(i-1,j).eq.-1).and.                      &     ! voisin west---------
                (x_gl_mx(i,j).lt.10.*dx).and.(x_gl_mx(i,j).ge.0.)) then 
              
              dH_x_voisin(1) = delHdx_mx(i,j)*retreat_x(i,j) 

              som_voisins  =  som_voisins + 1
              
           end if


           if ((mk_traiter(i+1,j).eq.-1).and.                      &     ! voisin East---------
                (x_gl_mx(i+1,j).gt.-10.*dx).and.(x_gl_mx(i+1,j).le.0.)) then 

               
              dH_x_voisin(2) = delHdx_mx(i+1,j)*retreat_x(i+1,j)
              som_voisins  =  som_voisins + 1

           end if


           if ((mk_traiter(i,j-1).eq.-1).and.                      &     ! voisin Sud---------
                (y_gl_my(i,j).lt.10.*dx).and.(y_gl_my(i,j).ge.0.)) then 

              dH_y_voisin(1) = delHdy_my(i,j)*retreat_y(i,j)
              som_voisins  =  som_voisins + 1

           end if

           if ((mk_traiter(i,j+1).eq.-1).and.                      &     ! voisin Nord---------
                (y_gl_my(i,j+1).gt.-10.*dx).and.(y_gl_my(i,j+1).le.0.)) then 

              dH_y_voisin(2) = delHdy_my(i,j+1)*retreat_y(i,j+1)
              som_voisins  =  som_voisins + 1

           endif


! en diagonale

           if ((mk_traiter(i-1,j-1).eq.-1).and.                     &     ! voisin SW---------
                (SW_gl_m(i,j).lt.10.*dx).and.(SW_gl_m(i,j).ge.0.)) then 
              
              dH_SW_voisin(1) = delHdx_SW(i,j)*retreat_SW(i,j) 

              som_voisins  =  som_voisins + 1
              
           end if


           if ((mk_traiter(i+1,j+1).eq.-1).and.                     &     ! voisin NE---------
                (SW_gl_m(i+1,j+1).gt.-10.*dx).and.(SW_gl_m(i+1,j+1).le.0.)) then 

              dH_SW_voisin(2) = delHdx_SW(i+1,j+1)*retreat_SW(i+1,j+1)
              som_voisins  =  som_voisins + 1

           end if


           if ((mk_traiter(i+1,j-1).le.-1).and.                     &     ! voisin SE---------
                (SE_gl_m(i,j).lt.10.*dx).and.(SE_gl_m(i,j).ge.0.)) then 

              dH_SE_voisin(1) = delHdx_SE(i,j)*retreat_SE(i,j)
              som_voisins  =  som_voisins + 1

           end if

           if ((mk_traiter(i-1,j+1).le.-1).and.                     &     ! voisin NW---------
                (SE_gl_m(i-1,j+1).gt.-10.*dx).and.(SE_gl_m(i-1,j+1).le.0.)) then 

              dH_SE_voisin(2) = delHdx_SE(i-1,j+1)*retreat_SE(i-1,j+1)
              som_voisins  =  som_voisins + 1

           endif

           if (som_voisins.eq.0) then
              write(6,'("probleme pour i,j=",11(i3,x))'), i,j,mk_traiter(i,j)
              write(6,*)'j-1',     mk_gr(i-1:i+1,j-1)
              write(6,*)'j  ',     mk_gr(i-1:i+1,j)
              write(6,*)'j+1',     mk_gr(i-1:i+1,j+1)
           end if

!     seuls les voisins au dessus peuvent venir
           call teste_socle_voisins(i,j,dH_x_voisin,dH_y_voisin,dH_SW_voisin,dH_SE_voisin)


!     calcul de la variation d'epaisseur pour ce point         

           max_x  = maxval(dH_x_voisin)
           max_y  = maxval(dH_y_voisin)
           max_SW = maxval(dH_SW_voisin)
           max_SE = maxval(dH_SE_voisin)
           delHdt(i,j) = max(max_x,max_y,max_SE,max_SW) 

! call sanity_check
           call sanity_check(i,j)
           delHdt(i,j) = min(delHdt(i,j),delHdt_sanity(i,j))

           H(i,j) = H(i,j)-delHdt(i,j)*dt                            ! attention peut etre sous-flottaison         


!      sanity check




        end if point_a_traiter
     end do
  end do
end subroutine calc_delHdt_maj



!-------------------------------------------------------------------------
!<  calc_H_float : calcule la hauteur de flottaison
!-------------------------------------------------------------------------

subroutine calc_H_float

  use declar_proto_recul
  implicit none

  H_float(:,:) = 0.

  where ((Mk_gr(:,:).gt.0).and.(Bsoc(:,:).lt.0.))     
     H_float(:,:)    = Bsoc(:,:) / Coef_bflot

  elsewhere ((Mk_gr(:,:).gt.0).and.(Bsoc(:,:).ge.0.))
     H_float(:,:) = -10000.

  elsewhere (Mk_gr(:,:).eq.0)
     H_float(:,:)    = Bsoc(:,:) / Coef_bflot

  end where

end subroutine calc_H_float



!-------------------------------------------------------------------------
!<  update_flottants :  points qui deviennent flottant
!-------------------------------------------------------------------------

subroutine update_flottants

  use declar_proto_recul

  implicit none

  integer                           :: som_voisins  !< pour des tests
  real                              :: Hf           !< Hf = 0 si socle > 0 sinon H_float
  real                              :: dyy         ! variable de travail  longueur de la maille en diagonale       

  dyy = dx*(2.**0.5)       !longueur de la diagonale

! update des masques : points qui deviennent flottant
  do j=2,ny-1
     do i=2,nx-1



        new_float: if ((mk_traiter(i,j).eq.1).and.(H(i,j).le.H_float(i,j))) then    ! le noeud est passe flottant
           mk_traiter(i,j) = -1
           time_float(i,j) = time
           H(i,j)          = H_float(i,j)-20.
           mk_gr(i,j)      = 0


           ! update vers les noeuds voisins

           if (mk_gr(i-1,j).eq.1)  then                              ! noeud west est grounded
              x_gl_mx(i,j)     = -(dx-1.)                            ! position gl proche de i,j
              Hf               = max(0.,H_float(i-1,j))
              delHdx_mx(i,j)   = ( H(i-1,j) - Hf )/dx                ! taux d'amincissement de (i-1,j)
           else
              x_gl_mx(i,j)     = -10.*dx
              delHdx_mx(i,j)   =   0.
           end if

           if (mk_gr(i+1,j).eq.1)  then                             ! noeud Est est grounded
              x_gl_mx(i+1,j)   = dx-1.                              ! position gl proche de i,j
              Hf               = max(0.,H_float(i+1,j))
              delHdx_mx(i+1,j) = ( H(i+1,j) - Hf )/dx               ! taux d'amincissement de (i+1,j)
           else
              x_gl_mx(i+1,j)   = -10.*dx
              delHdx_mx(i+1,j) =   0.
           end if

           if (mk_gr(i,j-1).eq.1)  then                              ! noeud Sud est grounded
              y_gl_my(i,j)     = -(dx-1.)                            ! position gl proche de i,j
              Hf               = max(0.,H_float(i,j-1))
              delHdy_my(i,j)   = ( H(i,j-1) - Hf )/dx                ! taux d'amincissement de (i-1,j)
           else
              y_gl_my(i,j)     = -10.*dx
              delHdy_my(i,j)   =   0.
           end if

           if (mk_gr(i,j+1).eq.1)  then                              ! noeud Nord est grounded
              y_gl_my(i,j+1)   = dx-1.                               ! position gl proche de i,j
              Hf               = max(0.,H_float(i,j+1))
              delHdy_my(i,j+1) = ( H(i,j+1) - Hf )/dx                ! taux d'amincissement de (i+1,j)
           else
              y_gl_my(i,j+1)   = -10.*dx
              delHdy_my(i,j+1) =   0.
           end if

! en diagonale
 
           if (mk_gr(i-1,j-1).eq.1)  then                            ! noeud SW grounded majeur (i-1,j-1)
              SW_gl_m(i,j)       = -(dyy-1.)                          ! position gl proche de i,j <0
              Hf                 = max(0.,H_float(i-1,j-1))          ! noeud mineur i,j
              delHdx_SW(i,j)     = ( H(i-1,j-1) - Hf )/dyy           ! taux d'amincissement de (i-1,j-1)
           else
              SW_gl_m(i,j)       = -10.*dx
              delHdx_SW(i,j)     =   0.
           end if

           if (mk_gr(i+1,j+1).eq.1)  then                            ! noeud NE grounded majeur (i+1,j+1)
              SW_gl_m(i+1,j+1)   = dyy-1.                             ! position gl proche de i,j
              Hf                 = max(0.,H_float(i+1,j+1))          ! noeud mineur (i+1,j+1)
              delHdx_SW(i+1,j+1) = ( H(i+1,j+1) - Hf )/dyy           ! taux d'amincissement de (i+1,j+1)
           else
              SW_gl_m(i+1,j+1)   = -10.*dx
              delHdx_SW(i+1,j+1) =   0.
           end if


           if (mk_gr(i+1,j-1).eq.1)  then                            ! noeud SE grounded majeur (i+1,j-1)
              SE_gl_m(i,j)       = -(dyy-1.)                         ! position gl proche de i,j <0
              Hf                 = max(0.,H_float(i+1,j-1))          ! noeud mineur i,j
              delHdx_SE(i,j)     = ( H(i+1,j-1) - Hf )/dyy           ! taux d'amincissement de (i+1,j-1)

           else
              SE_gl_m(i,j)       = -10.*dx
              delHdx_SE(i,j)     =   0.
           end if

           if (mk_gr(i-1,j+1).eq.1)  then                            ! noeud NW grounded majeur (i-1,j+1)
              SE_gl_m(i-1,j+1)   = dyy-1.                            ! position gl proche de i,j
              Hf                 = max(0.,H_float(i-1,j+1))          ! noeud mineur (i-1,j+1)
              delHdx_SE(i-1,j+1) = ( H(i-1,j+1) - Hf )/dyy           ! taux d'amincissement de (i-1,j+1)
           else
              SE_gl_m(i-1,j+1)   = -10.*dx
              delHdx_SE(i-1,j+1) =   0.
           end if

            
        end if new_float
     end do
  end do

end subroutine update_flottants



!-------------------------------------------------------------------------
!<  update_a_traiter :  points qui deviennent a traiter
!-------------------------------------------------------------------------

subroutine update_a_traiter

  use declar_proto_recul
  implicit none

  integer                           :: som_voisins  !< pour des tests

  ! les nouveaux points a traiter
  do j=2,ny-1
     do i =2,nx-1

!       formulation en croix
!        som_voisins = mk_gr(i-1,j)+mk_gr(i+1,j)+mk_gr(i,j-1)+mk_gr(i,j+1)      

!        if ((mk_traiter(i,j).eq.2).and.(som_voisins.gt.0).and.(som_voisins.lt.4)) then
!           mk_traiter(i,j) = 1
!        end if

!       Formulation avec diagonales 
       if ((mk_traiter(i,j).eq.2).and.(any(mk_gr(i-1:i+1,j-1:j+1).eq.0))) then
          mk_traiter(i,j) = 1
       end if

     end do
  end do
end subroutine update_a_traiter

!-------------------------------------------------------------------------
!<  prescribe_Hp :  renvoie Hp et I_Hp vers GRISLI
!-------------------------------------------------------------------------

subroutine prescribe_Hp

  use declar_proto_recul
  implicit none

! remet les masques Hp et i_Hp aux valeurs fixes

      i_HP(:,:) = i_Hp0(:,:)
      Hp(:,:)   = Hp0(:,:)

i=208
j=176

  where (mk_gr(:,:).eq.0)       ! points flottants
     i_Hp(:,:) = 1
     Hp (:,:)  = H(:,:)         ! les points flottants ne sont pas modifies
  end where

  where(mk_traiter(:,:).eq.1)   ! points imposes Epaisseur en cours de decroissance
     i_Hp(:,:) = 1
     Hp (:,:)  = H(:,:)        
  end where

end subroutine prescribe_Hp

!-------------------------------------------------------------------------
!<  calc_eps_max :  calcule le epsilon max (sanity check)
!-------------------------------------------------------------------------

subroutine calc_eps_max  

  use declar_proto_recul
  implicit none
  real              :: gamma   !< coefficient de flottaison
  
  gamma = ro*g*(1.+coef_Bflot)

  epsmax(:,:) = Abar(:,:)*H(:,:)**3*(gamma/4.)**3

end subroutine calc_eps_max

!-------------------------------------------------------------------------
!<  calc_xgl :  calcule la position sous maille de la grounding line
!<              en supposant que l'epaisseur varie lineairement
!-------------------------------------------------------------------------

subroutine calc_xgl(dy,alpha,H_0,B_0,H_1,B_1,xpos,delHdx)  

  implicit none

! dummy
  real,intent(in)        ::    dy        !< longueur de la maille 
  real,intent(in)        ::    alpha     !< coefficient de flottaison = coef_Bflot
  real,intent(in)        ::    H_0       !< epaisseur au point pose
  real,intent(in)        ::    B_0       !< altitude socle au point pose
  real,intent(in)        ::    H_1       !< epaisseur au point flottant
  real,intent(in)        ::    B_1       !< altitude socle au point flottant
  real,intent(out)       ::    xpos      !< position de la ligne (en distance depuis le point pose  
  real,intent(out)       ::    delHdx    !< variation d'epaisseur au noeud pose en fonction d'une variation de xpos     
  

  real                   ::    Cgl       ! variable de travail  

  Cgl   = (alpha * (H_1-H_0) - (B_1-B_0)) 

  if (abs(Cgl).gt.1.e-5) then   
     xpos    = dy * (B_0 - alpha * H_0) / Cgl 
  else
     xpos    = dy-1.   ! verifier
  end if

! la variation d'epaisseur est proportionnelle au recul

  if (xpos.gt.1.) then
     delHdx = (H_0 - B_0 /alpha) / xpos 
  else
     delHdx = (H_0 - B_0 /alpha)
  end if

end subroutine calc_xgl

!----------------------------------------------------------------------
!  lect_retreat_files : lit les fichiers specifiques au retrait
!----------------------------------------------------------------------
subroutine  lect_retreat_files

use netcdf
use io_netcdf_grisli
use declar_proto_recul

implicit none

integer    :: num               ! pour la lecture
integer    :: num_job           ! numero du job
real*8, dimension(:,:),   pointer  :: tab              !< tableau 2d real ecrit dans le fichier

character(len=2) :: char_num    ! pour la lecture

! lit le numero du job

char_num = runname(7:8)
read(char_num,*) num_job
write(6,*) num_job

! noms des fichiers d'entree

number_data_B_file = trim(dirnameinp)//trim(number_data_B_file)
region_file        = trim(dirnameinp)//trim(region_file)
file_exp           = trim(dirnameinp)//trim(file_exp)


! lit le fichier sigma_socle
call Read_Ncdf_var('z',number_data_B_file,tab)      ! lit la variable 'z'
Bsoc_sigma(:,:)  = tab(:,:)

! lit le fichier region
call Read_Ncdf_var('z',region_file,tab)             ! lit la variable 'z'
map_region(:,:)  = nint(tab(:,:))


! lit le fichier d'experience pour ne garder que la ligne concernee

open(99,file=file_exp)
read(99,*)                                               ! lit la ligne de titre

do i=1,200
   read(99,*,end=200) num, beta_lim_ret,retreat_1,retreat_2,(time_region(j),j=1,nb_regions)
   if (num.eq.num_job) exit
end do

write(6,*) 'runname ',runname,'   num',num
write(6,*) 'beta_lim,retrat1,retreat2', beta_lim_ret,retreat_1,retreat_2
write(6,*) 'time',time_region(:)
write(6,*) '----------------------------------------------------------'

return
200 write(6,*) " ce numero de job n est pas dans le fichier"



end subroutine lect_retreat_files



!----------------------------------------------------------------------
!  init_retreat_rates : calcule les taux de retrait en fonction de divers criteres
!----------------------------------------------------------------------
subroutine  init_retreat0

use declar_proto_recul
implicit none

! critere sur le beta 
!------------------------

travail_centre(:,:) =  beta_centre(:,:)

call moyennes_demi_mailles 

where (travail_mx(:,:).gt.beta_lim_ret)       !---- en x
   retreat0_x(:,:) = retreat_2*1000.
elsewhere
   retreat0_x(:,:) = retreat_1*1000.
end where

where (travail_my(:,:).gt.beta_lim_ret)       !---- en y
   retreat0_y(:,:) = retreat_2*1000.
elsewhere
   retreat0_y(:,:) = retreat_1*1000.
end where

where (travail_SW(:,:).gt.beta_lim_ret)      !---- en SW
   retreat0_SW(:,:) = retreat_2*1000.
elsewhere
   retreat0_SW(:,:) = retreat_1*1000.
end where

where (travail_SE(:,:).gt.beta_lim_ret)      !---- en SE
   retreat0_SE(:,:) = retreat_2*1000.
elsewhere
   retreat0_SE(:,:) = retreat_1*1000.
end where

! enleve les regions qui ne sont pas a traiter

  do j=2,ny-1
     do i =2,nx-1
        if (mk_traiter(i,j)*mk_traiter(i-1,j).eq.0) then     !---- en x
           retreat0_x(i,j)  = 0.
        end if
        if (mk_traiter(i,j)*mk_traiter(i,j-1).eq.0) then     !---- en y
           retreat0_y(i,j)  = 0.
        end if
        if (mk_traiter(i,j)*mk_traiter(i-1,j-1).eq.0) then   !---- en SW
           retreat0_SW(i,j) = 0.
        end if
        if (mk_traiter(i,j)*mk_traiter(i+1,j-1).eq.0) then   !---- en SE
           retreat0_SE(i,j) = 0.
        end if
     end do
  end do


end subroutine init_retreat0


!----------------------------------------------------------------------
!  moyennes_demi_mailles : calcule les moyennes d'un tableau ! travail

!----------------------------------------------------------------------
subroutine moyennes_demi_mailles 
use declar_proto_recul
implicit none
  do j=2,ny-1
     do i =2,nx-1
        travail_mx(i,j) = 0.5* ( travail_centre(i,j)  + travail_centre(i-1,j) )
        travail_my(i,j) = 0.5* ( travail_centre(i,j)  + travail_centre(i,j-1) )
        travail_SW(i,j) = 0.5* ( travail_centre(i,j)  +travail_centre(i-1,j-1))
        travail_SE(i,j) = 0.5* ( travail_centre(i,j)  +travail_centre(i+1,j-1))
     end do
  end do

end subroutine moyennes_demi_mailles

!----------------------------------------------------------------------
!  init_time_depart : donne les temps de depart en fonction des regions
!----------------------------------------------------------------------

subroutine init_time_depart


use declar_proto_recul
implicit none
real               :: time_inacc      = 10000.   ! temps donne aux noeuds inaccessibles
real               :: t_already_float = -10.     ! temps donne aux noeuds qui flottent deja
real               :: time_run_begin  = 2000.    ! debut du run              


! region par region

  time_region(0) =  t_already_float-time_run_begin

  do j=2,ny-1
     do i=2,nx-1
        time_dep(i,j) = (time_region(map_region(i,j)) - time_run_begin)
     end do
  end do

  where (mk_traiter(:,:).eq.0)
     time_dep(:,:) = time_inacc
  end where

! calcul sur les demi-mailles
  travail_centre(:,:) = time_dep(:,:)

  call moyennes_demi_mailles

  time_dep_mx(:,:) = travail_mx(:,:)
  time_dep_my(:,:) = travail_my(:,:)
  time_dep_SW(:,:) = travail_SW(:,:)
  time_dep_SE(:,:) = travail_SE(:,:)


! le temps est tres eleve pour les zones inaccessibles
  do j=2,ny-1
     do i =2,nx-1
        if (mk_traiter(i,j)*mk_traiter(i-1,j).eq.0) then     !---- en x
           time_dep_mx(i,j)  = time_inacc
        end if
        if (mk_traiter(i,j)*mk_traiter(i,j-1).eq.0) then     !---- en y
           time_dep_my(i,j)  = time_inacc
        end if
        if (mk_traiter(i,j)*mk_traiter(i-1,j-1).eq.0) then   !---- en SW
           time_dep_SW(i,j) = time_inacc
        end if
        if (mk_traiter(i,j)*mk_traiter(i+1,j-1).eq.0) then   !---- en SE
           time_dep_SE(i,j) = time_inacc
        end if
     end do
  end do

! pour bien voir les rgions dans time_float
  where(mk_traiter(:,:).eq.2)
     time_float(:,:) = tend + 100.
  elsewhere (mk_traiter(:,:).eq.0)
     time_float(:,:) = time_inacc
  elsewhere (mk_traiter(:,:).lt.0)
     time_float(:,:) = t_already_float
  end where


end subroutine init_time_depart

!----------------------------------------------------------------------
!<  bedrock_sigma : calcul la tolerance sur le socle
!                   en fonction de bed_sigma_1 et bed_sigma_2!                  
!----------------------------------------------------------------------

subroutine  bedrock_sigma


use declar_proto_recul
implicit none

real        :: prec_min = 20.          ! precision min de la mesure


where (Bsoc_sigma (:,:).lt.1.)         ! pas de donnee socle
   travail_centre(:,:) = bed_sigma_2
elsewhere (Bsoc_sigma (:,:).ge.1.)     ! 
   travail_centre(:,:) = max(prec_min,bed_sigma_1/sqrt(Bsoc_sigma(:,:)))
end where
Bsoc_sigma (:,:) = travail_centre(:,:)

! zones particulieres

where(mk_traiter(:,:).eq.0) 
   Bsoc_sigma(:,:) = -10000            ! zones interdites
elsewhere (mk_traiter(:,:).lt.0)
   Bsoc_sigma(:,:) = 0.
end where
   
end subroutine bedrock_sigma


!----------------------------------------------------------------------
!< teste_socle_voisins : pour un point i,j : teste si les voisins sont plus haut                           
!----------------------------------------------------------------------

subroutine teste_socle_voisins(ii,jj,A_x,A_y,A_SW,A_SE)

use declar_proto_recul
implicit none

integer,intent(in)                 :: ii,jj         ! indice du point considere
real,dimension(2),intent(inout)    :: A_x           ! quand on balaye les voisins
real,dimension(2),intent(inout)    :: A_y           ! quand on balaye les voisins
real,dimension(2),intent(inout)    :: A_SW          ! quand on balaye les voisins
real,dimension(2),intent(inout)    :: A_SE          ! quand on balaye les voisins



! Btest est plus haut = Bsoc + Bsoc_sigma


! Mais pas suffisamment

if (Btest(ii-1,jj  ).le.Bsoc(ii,jj))  A_x(1) = 0.  !---- en x
if (Btest(ii+1,jj  ).le.Bsoc(ii,jj))  A_x(2) = 0.
if (Btest(ii  ,jj-1).le.Bsoc(ii,jj))  A_y(1) = 0.  !---- en y
if (Btest(ii  ,jj+1).le.Bsoc(ii,jj))  A_y(2) = 0.

if (Btest(ii-1,jj-1).le.Bsoc(ii,jj))  A_SW(1) = 0. !---- en SW
if (Btest(ii+1,jj+1).le.Bsoc(ii,jj))  A_SW(2) = 0.
if (Btest(ii+1,jj-1).le.Bsoc(ii,jj))  A_SE(1) = 0. !---- en SE
if (Btest(ii-1,jj+1).le.Bsoc(ii,jj))  A_SE(2) = 0.

end subroutine teste_socle_voisins


!----------------------------------------------------------------------
!< sanity_check : pour un point i,j  limite le dhdt                         
!----------------------------------------------------------------------

subroutine sanity_check(ii,jj)

use declar_proto_recul
implicit none

integer,intent(in)                 :: ii,jj         ! indice du point considere

real                               :: dH_x          ! variable de travail
real                               :: dH_y          ! variable de travail

DelHdt_sanity(ii,jj) = 0.

! en x -----------------------------------------------------------------------------------
if ((Uxbar(ii,jj).ge.0.).and.(Uxbar(ii+1,jj).ge.0.)) then             ! ecoulement --->

   dH_x   =  Uxbar(ii,jj)*(H(ii,jj)-H(ii-1,jj)) / dx
   dH_x   =  Dh_x +  epsmax(ii,jj) * H(ii,jj)

else if ((Uxbar(ii,jj).le.0.).and.(Uxbar(ii+1,jj).le.0.)) then        ! ecoulement <---

   dH_x  =  Uxbar(ii+1,jj)*(H(ii+1,jj)-H(ii,jj)) / dx
   dH_x  =  Dh_x +  epsmax(ii,jj) * H(ii,jj)

else if ((Uxbar(ii,jj).le.0.).and.(Uxbar(ii+1,jj).ge.0.)) then        ! ecoulement  <------>

   dH_x   =  Dh_x +  epsmax(ii,jj) * H(ii,jj)
else                                                                  ! ecoulement ---><-----
   dH_x  =  Uxbar(ii,jj)*(H(ii,jj)-H(ii-1,jj)) / dx + Uxbar(ii+1,jj)*(H(ii+1,jj)-H(ii,jj)) / dx
!  mais ici pas de terme en epsmax car confine
endif


! en y -----------------------------------------------------------------------------------
  if ((Uybar(ii,jj).ge.0.).and.(Uybar(ii,jj+1).ge.0.)) then           ! ecoulement  ^
                                                       
   dH_y   =  Uybar(ii,jj)*(H(ii,jj)-H(ii,jj-1)) / dx
   dH_y   =  Dh_y +  epsmax(ii,jj) * H(ii,jj)

else if ((Uybar(ii,jj).le.0.).and.(Uybar(ii,jj+1).le.0.)) then        ! ecoulement  v

   dH_y  =  Uybar(ii,jj+1)*(H(ii,jj+1)-H(ii,jj)) / dx
   dH_y  =  Dh_y +  epsmax(ii,jj) * H(ii,jj)

else if ((Uybar(ii,jj).le.0.).and.(Uybar(ii,jj+1).ge.0.)) then        ! ecoulement divergent

   dH_y   =  Dh_y +  epsmax(ii,jj) * H(ii,jj)
else                                                                  ! ecoulement convergent
   dH_y  =  Uybar(ii,jj)*(H(ii,jj)-H(ii,jj-1)) / dx + Uybar(ii,jj+1)*(H(ii,jj+1)-H(ii,jj)) / dx
!  mais ici pas de terme en epsmax car confine
endif

DelHdt_sanity(ii,jj) = max(dH_x + dH_y,0.)

end subroutine sanity_check

end module proto_recul_mod
!----------------------------------------------------------------------------------





